@@ -1,33 +1,12 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "crumblebar.h"
 
 #include "qmldesignerplugin.h"
 
 #include <nodeabstractproperty.h>
+#include <toolbar.h>
 
 #include <coreplugin/documentmanager.h>
 #include <coreplugin/idocument.h>
@@ -63,23 +42,14 @@ static CrumbleBarInfo createCrumbleBarInfoFromModelNode(const ModelNode &modelNo
 {
     CrumbleBarInfo crumbleBarInfo;
     crumbleBarInfo.displayName = componentIdForModelNode(modelNode);
-    crumbleBarInfo.fileName = currentDesignDocument()->textEditor()->document()->filePath().toString();
+    crumbleBarInfo.fileName = currentDesignDocument()->textEditor()->document()->filePath();
     crumbleBarInfo.modelNode = modelNode;
 
     return crumbleBarInfo;
 }
 
-CrumbleBar::CrumbleBar(QObject *parent) :
-    QObject(parent),
-    m_isInternalCalled(false),
-    m_crumblePath(new Utils::CrumblePath)
+CrumbleBar::CrumbleBar(QObject *parent) : QObject(parent)
 {
-    connect(m_crumblePath,
-            SIGNAL(elementClicked(QVariant)),
-            this,
-            SLOT(onCrumblePathElementClicked(QVariant)));
-
-    updateVisibility();
 }
 
 CrumbleBar::~CrumbleBar()
@@ -87,26 +57,39 @@ CrumbleBar::~CrumbleBar()
     delete m_crumblePath;
 }
 
-void CrumbleBar::pushFile(const QString &fileName)
+void CrumbleBar::pushFile(const Utils::FilePath &fileName)
 {
-    if (m_isInternalCalled == false) {
+    if (!m_isInternalCalled) {
         crumblePath()->clear();
+        m_pathes.clear();
     } else {
-        CrumbleBarInfo lastElementCrumbleBarInfo = crumblePath()->dataForLastIndex().value<CrumbleBarInfo>();
+        // If the path already exists in crumblePath, pop up to first instance of that to avoid
+        // cyclical crumblePath
+        int match = -1;
+        for (int i = crumblePath()->length() - 1; i >= 0; --i) {
+            CrumbleBarInfo info = crumblePath()->dataForIndex(i).value<CrumbleBarInfo>();
+            if (info.fileName == fileName)
+                match = i;
+        }
 
-        if (!lastElementCrumbleBarInfo.displayName.isEmpty()
-                && lastElementCrumbleBarInfo.fileName == fileName)
-            crumblePath()->popElement();
+        if (match != -1) {
+            for (int i = crumblePath()->length() - 1 - match; i > 0; --i)
+                popElement();
+        }
     }
 
-    CrumbleBarInfo crumbleBarInfo;
-    crumbleBarInfo.fileName = fileName;
-
-    crumblePath()->pushElement(fileName.split(QLatin1String("/")).last(), QVariant::fromValue(crumbleBarInfo));
+    CrumbleBarInfo info = crumblePath()->dataForLastIndex().value<CrumbleBarInfo>();
+    if (info.fileName != fileName) {
+        CrumbleBarInfo crumbleBarInfo;
+        crumbleBarInfo.fileName = fileName;
+        crumblePath()->pushElement(fileName.fileName(), QVariant::fromValue(crumbleBarInfo));
+        m_pathes.append({fileName, fileName.fileName(), {}});
+    }
 
     m_isInternalCalled = false;
-
     updateVisibility();
+
+    emit pathChanged();
 }
 
 void CrumbleBar::pushInFileComponent(const ModelNode &modelNode)
@@ -119,10 +102,13 @@ void CrumbleBar::pushInFileComponent(const ModelNode &modelNode)
         crumblePath()->popElement();
 
     crumblePath()->pushElement(crumbleBarInfo.displayName, QVariant::fromValue(crumbleBarInfo));
+    m_pathes.append({{}, crumbleBarInfo.displayName, modelNode});
 
     m_isInternalCalled = false;
 
     updateVisibility();
+
+    emit pathChanged();
 }
 
 void CrumbleBar::nextFileIsCalledInternally()
@@ -132,54 +118,87 @@ void CrumbleBar::nextFileIsCalledInternally()
 
 Utils::CrumblePath *CrumbleBar::crumblePath()
 {
+    if (m_crumblePath == nullptr) {
+        m_crumblePath = new Utils::CrumblePath;
+        updateVisibility();
+        connect(m_crumblePath, &Utils::CrumblePath::elementClicked,
+            this, &CrumbleBar::onCrumblePathElementClicked);
+    }
+
     return m_crumblePath;
 }
 
-void CrumbleBar::showSaveDialog()
+QStringList CrumbleBar::path() const
 {
-    DesignerSettings settings = QmlDesignerPlugin::instance()->settings();
+    QStringList list;
+    for (auto &path : m_pathes) {
+        list.append(path.displayName);
+    }
 
-    if (settings.value(DesignerSettingsKey::ALWAYS_SAFE_IN_CRUMBLEBAR).toBool()) {
+    return list;
+}
+
+QList<CrumbleBarInfo> CrumbleBar::infos() const
+{
+    return m_pathes;
+}
+
+bool CrumbleBar::showSaveDialog()
+{
+    bool canceled = false;
+    bool alwaysSave = QmlDesignerPlugin::settings().value(DesignerSettingsKey::ALWAYS_SAVE_IN_CRUMBLEBAR).toBool();
+    if (alwaysSave) {
         Core::DocumentManager::saveModifiedDocumentSilently(currentDesignDocument()->editor()->document());
     } else {
-        bool alwaysSave;
-        bool canceled;
-
         Core::DocumentManager::saveModifiedDocument(currentDesignDocument()->editor()->document(),
                                                     tr("Save the changes to preview them correctly."),
                                                     &canceled,
                                                     tr("Always save when leaving subcomponent"),
                                                     &alwaysSave);
 
-        settings.insert(DesignerSettingsKey::ALWAYS_SAFE_IN_CRUMBLEBAR, alwaysSave);
-        QmlDesignerPlugin::instance()->setSettings(settings);
+        QmlDesignerPlugin::settings().insert(DesignerSettingsKey::ALWAYS_SAVE_IN_CRUMBLEBAR, alwaysSave);
     }
+    return !canceled;
+}
+
+void CrumbleBar::popElement()
+{
+    crumblePath()->popElement();
+
+    if (!m_pathes.isEmpty())
+        m_pathes.removeLast();
 }
 
 void CrumbleBar::onCrumblePathElementClicked(const QVariant &data)
 {
     CrumbleBarInfo clickedCrumbleBarInfo = data.value<CrumbleBarInfo>();
 
-    if (clickedCrumbleBarInfo ==  crumblePath()->dataForLastIndex().value<CrumbleBarInfo>())
+    if (clickedCrumbleBarInfo == crumblePath()->dataForLastIndex().value<CrumbleBarInfo>())
         return;
 
-    while (clickedCrumbleBarInfo != crumblePath()->dataForLastIndex().value<CrumbleBarInfo>())
-        crumblePath()->popElement();
+    bool inlineComp = !clickedCrumbleBarInfo.modelNode.isValid()
+            && clickedCrumbleBarInfo.fileName == currentDesignDocument()->fileName();
+
+    if (!inlineComp && !showSaveDialog())
+        return;
+
+    while (clickedCrumbleBarInfo != crumblePath()->dataForLastIndex().value<CrumbleBarInfo>()
+           && crumblePath()->length() > 0)
+        popElement();
 
     if (crumblePath()->dataForLastIndex().value<CrumbleBarInfo>().modelNode.isValid())
-        crumblePath()->popElement();
+        popElement();
 
     m_isInternalCalled = true;
-    if (!clickedCrumbleBarInfo.modelNode.isValid()
-            && Utils::FileName::fromString(clickedCrumbleBarInfo.fileName) == currentDesignDocument()->fileName()) {
+    if (inlineComp) {
         nextFileIsCalledInternally();
         currentDesignDocument()->changeToDocumentModel();
         QmlDesignerPlugin::instance()->viewManager().setComponentViewToMaster();
     } else {
-        showSaveDialog();
-        crumblePath()->popElement();
+        popElement();
         nextFileIsCalledInternally();
-        Core::EditorManager::openEditor(clickedCrumbleBarInfo.fileName, Core::Id(),
+        Core::EditorManager::openEditor(clickedCrumbleBarInfo.fileName,
+                                        Utils::Id(),
                                         Core::EditorManager::DoNotMakeVisible);
         if (clickedCrumbleBarInfo.modelNode.isValid()) {
             currentDesignDocument()->changeToSubComponent(clickedCrumbleBarInfo.modelNode);
@@ -188,12 +207,14 @@ void CrumbleBar::onCrumblePathElementClicked(const QVariant &data)
             QmlDesignerPlugin::instance()->viewManager().setComponentViewToMaster();
         }
     }
+    emit pathChanged();
     updateVisibility();
 }
 
 void CrumbleBar::updateVisibility()
 {
-    crumblePath()->setVisible(crumblePath()->length() > 1);
+    if (!ToolBar::isVisible())
+        crumblePath()->setVisible(crumblePath()->length() > 1);
 }
 
 bool operator ==(const CrumbleBarInfo &first, const CrumbleBarInfo &second)
